/*----------------------------------------------------------------------------
 MINIGUI - Harbour Win32 GUI library source code

 Copyright 2002-2010 Roberto Lopez <harbourminigui@gmail.com>
 http://harbourminigui.googlepages.com/

 This program is free software; you can redistribute it and/or modify it under 
 the terms of the GNU General Public License as published by the Free Software 
 Foundation; either version 2 of the License, or (at your option) any later 
 version. 

 This program is distributed in the hope that it will be useful, but WITHOUT 
 ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS 
 FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.

 You should have received a copy of the GNU General Public License along with 
 this software; see the file COPYING. If not, write to the Free Software 
 Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA (or 
 visit the web site http://www.gnu.org/).

 As a special exception, you have permission for additional uses of the text 
 contained in this release of Harbour Minigui.

 The exception is that, if you link the Harbour Minigui library with other 
 files to produce an executable, this does not by itself cause the resulting 
 executable to be covered by the GNU General Public License.
 Your use of that executable is in no way restricted on account of linking the 
 Harbour-Minigui library code into it.

 Parts of this project are based upon:

	"Harbour GUI framework for Win32"
 	Copyright 2001 Alexander S.Kresin <alex@kresin.ru>
 	Copyright 2001 Antonio Linares <alinares@fivetech.com>
	www - https://harbour.github.io/

	"Harbour Project"
	Copyright 1999-2023, https://harbour.github.io/

	"WHAT32"
	Copyright 2002 AJ Wos <andrwos@aust1.net> 

	"HWGUI"
  	Copyright 2001-2021 Alexander S.Kresin <alex@kresin.ru>

---------------------------------------------------------------------------*/

#command CREATE LINK FILE <fname> ;
   TARGETFILE <target> ;
   [ TARGETARGS <args> ] ;
   [ DESCRIPTION <desc> ] ;
   [ SHOWMODE <nMode> ] ;
   [ [ WORKING ] DIRECTORY <curdir> ] ;
   [ ICON [ LOCATION ] <iconfile> ] ;
   [ ICONINDEX <iconidx> ] ;
   [ HOTKEY <key> ] ;
   [ RESULT [ TO ] <nSuccess> ] ;
=> ;
   [ <nSuccess> := ] HMG_CreateLink( <target>, <args>, <fname>, <desc>, ;
                                 <nMode>, <curdir>, <iconfile>, <iconidx>, <"key"> )

#command INSERT BEFORE           => dbInsert()
#command INSERT BLANK            => dbInsert( RecNo() + 1 )

// Registry functions for Harbour defines
// Contributed by Luiz Rafael Culik<culik@sl.conex.net>

#define  HKEY_CLASSES_ROOT       2147483648       // 0x80000000
#define  HKEY_CURRENT_USER       2147483649       // 0x80000001
#define  HKEY_LOCAL_MACHINE      2147483650       // 0x80000002
#define  HKEY_USERS              2147483651       // 0x80000003
#define  HKEY_PERFORMANCE_DATA   2147483652       // 0x80000004
#define  HKEY_CURRENT_CONFIG     2147483653       // 0x80000005
#define  HKEY_DYN_DATA           2147483654       // 0x80000006

// The following are masks for the predefined standard access types

#define SYNCHRONIZE             1048576    // 0x00100000L
#define STANDARD_RIGHTS_READ    131072     // 0x00020000L
#define STANDARD_RIGHTS_WRITE   131072     // 0x00020000L
#define STANDARD_RIGHTS_ALL     2031616    // 0x001F0000L

// Registry Specific Access Rights.

#define KEY_QUERY_VALUE         1    // 0x0001
#define KEY_SET_VALUE           2    // 0x0002
#define KEY_CREATE_SUB_KEY      4    // 0x0004
#define KEY_ENUMERATE_SUB_KEYS  8    // 0x0008
#define KEY_NOTIFY              16   // 0x0010
#define KEY_CREATE_LINK         32   // 0x0020

***
*** Not sure how to handle the &'s and ~'s
***

#define KEY_READ        25 // ((STANDARD_RIGHTS_READ +  KEY_QUERY_VALUE + KEY_ENUMERATE_SUB_KEYS +  KEY_NOTIFY) & (~SYNCHRONIZE))
#define KEY_WRITE        6 // ((STANDARD_RIGHTS_WRITE +  KEY_SET_VALUE +  KEY_CREATE_SUB_KEY) & (~SYNCHRONIZE))
#define KEY_EXECUTE     25 // ((KEY_READ) & (~SYNCHRONIZE))
#define KEY_ALL_ACCESS  63 // ((STANDARD_RIGHTS_ALL +  KEY_QUERY_VALUE +  KEY_SET_VALUE +  KEY_CREATE_SUB_KEY + KEY_ENUMERATE_SUB_KEYS +  KEY_NOTIFY +  KEY_CREATE_LINK) & (~SYNCHRONIZE))

// Predefined Value Types.

#define REG_SZ                          1       && Data string (unicode nul terminated)
#define REG_EXPAND_SZ                   2       && Unicode string
#define REG_BINARY                      3       && Binary data in any form.
#define REG_DWORD                       4       && A 32-bit number.

// End Registry functions for Harbour defines

// browsing folders (Contributed by Richard Rylco)
#define BIF_RETURNONLYFSDIRS   0x0001       // For finding a folder to start document searching
#define BIF_DONTGOBELOWDOMAIN  0x0002       // For starting the Find Computer
#define BIF_STATUSTEXT         0x0004       // Top of the dialog has 2 lines of text for BROWSEINFO.lpszTitle and one line if
                                            // this flag is set.  Passing the message BFFM_SETSTATUSTEXTA to the hwnd can set the
                                            // rest of the text.  This is not used with BIF_USENEWUI and BROWSEINFO.lpszTitle gets
                                            // all three lines of text.
#define BIF_RETURNFSANCESTORS  0x0008
#define BIF_EDITBOX            0x0010       // Add an editbox to the dialog
#define BIF_VALIDATE           0x0020       // insist on valid result (or CANCEL)
#define BIF_NEWDIALOGSTYLE     0x0040       // Use the new dialog layout with the ability to resize
                                            // Caller needs to call OleInitialize() before using this API
#define BIF_USENEWUI           hb_BitOr( BIF_NEWDIALOGSTYLE, BIF_EDITBOX )
#define BIF_BROWSEINCLUDEURLS  0x0080       // Allow URLs to be displayed or entered. (Requires BIF_USENEWUI)
#define BIF_BROWSEFORCOMPUTER  0x1000       // Browsing for Computers.
#define BIF_BROWSEFORPRINTER   0x2000       // Browsing for Printers
#define BIF_BROWSEINCLUDEFILES 0x4000       // Browsing for Everything
#define BIF_SHAREABLE          0x8000       // sharable resources displayed (remote shares, requires BIF_USENEWUI)
#define BIF_NONEWFOLDERBUTTON  0x00000200   // Do not add the "New Folder" button to the dialog.  Only applicable with BIF_NEWDIALOGSTYLE.

// Special folders (Contributed by Richard Rylco)
#define CSIDL_DESKTOP                   0x0000        // <desktop>
#define CSIDL_INTERNET                  0x0001        // Internet Explorer (icon on desktop)
#define CSIDL_PROGRAMS                  0x0002        // Start Menu\Programs
#define CSIDL_CONTROLS                  0x0003        // My Computer\Control Panel
#define CSIDL_PRINTERS                  0x0004        // My Computer\Printers
#define CSIDL_PERSONAL                  0x0005        // My Documents
#define CSIDL_FAVORITES                 0x0006        // <user name>\Favorites
#define CSIDL_STARTUP                   0x0007        // Start Menu\Programs\Startup
#define CSIDL_RECENT                    0x0008        // <user name>\Recent
#define CSIDL_SENDTO                    0x0009        // <user name>\SendTo
#define CSIDL_BITBUCKET                 0x000a        // <desktop>\Recycle Bin
#define CSIDL_STARTMENU                 0x000b        // <user name>\Start Menu
#define CSIDL_DESKTOPDIRECTORY          0x0010        // <user name>\Desktop
#define CSIDL_DRIVES                    0x0011        // My Computer
#define CSIDL_NETWORK                   0x0012        // Network Neighborhood
#define CSIDL_NETHOOD                   0x0013        // <user name>\nethood
#define CSIDL_FONTS                     0x0014        // windows\fonts
#define CSIDL_TEMPLATES                 0x0015
#define CSIDL_COMMON_STARTMENU          0x0016        // All Users\Start Menu
#define CSIDL_COMMON_PROGRAMS           0X0017        // All Users\Programs
#define CSIDL_COMMON_STARTUP            0x0018        // All Users\Startup
#define CSIDL_COMMON_DESKTOPDIRECTORY   0x0019        // All Users\Desktop
#define CSIDL_APPDATA                   0x001a        // <user name>\Application Data
#define CSIDL_PRINTHOOD                 0x001b        // <user name>\PrintHood
#define CSIDL_LOCAL_APPDATA             0x001c        // <user name>\Local Settings\Applicaiton Data (non roaming)
#define CSIDL_ALTSTARTUP                0x001d        // non localized startup
#define CSIDL_COMMON_ALTSTARTUP         0x001e        // non localized common startup
#define CSIDL_COMMON_FAVORITES          0x001f
#define CSIDL_INTERNET_CACHE            0x0020
#define CSIDL_COOKIES                   0x0021
#define CSIDL_HISTORY                   0x0022
#define CSIDL_COMMON_APPDATA            0x0023        // All Users\Application Data
#define CSIDL_WINDOWS                   0x0024        // GetWindowsDirectory()
#define CSIDL_SYSTEM                    0x0025        // GetSystemDirectory()
#define CSIDL_PROGRAM_FILES             0x0026        // C:\Program Files
#define CSIDL_MYPICTURES                0x0027        // C:\Program Files\My Pictures
#define CSIDL_PROFILE                   0x0028        // USERPROFILE
#define CSIDL_SYSTEMX86                 0x0029        // x86 system directory on RISC
#define CSIDL_PROGRAM_FILESX86          0x002a        // x86 C:\Program Files on RISC
#define CSIDL_PROGRAM_FILES_COMMON      0x002b        // C:\Program Files\Common
#define CSIDL_PROGRAM_FILES_COMMONX86   0x002c        // x86 Program Files\Common on RISC
#define CSIDL_COMMON_TEMPLATES          0x002d        // All Users\Templates
#define CSIDL_COMMON_DOCUMENTS          0x002e        // All Users\Documents
#define CSIDL_COMMON_ADMINTOOLS         0x002f        // All Users\Start Menu\Programs\Administrative Tools
#define CSIDL_ADMINTOOLS                0x0030        // <user name>\Start Menu\Programs\Administrative Tools
#define CSIDL_CONNECTIONS               0x0031        // Network and Dial-up Connections

#define CSIDL_FLAG_CREATE               0x8000        // combine with CSIDL_ value to force folder creation in SHGetFolderPath()
#define CSIDL_FLAG_DONT_VERIFY          0x4000        // combine with CSIDL_ value to return an unverified folder path
#define CSIDL_FLAG_MASK                 0xFF00        // mask for all possible flag values

